<?php

namespace ModulesGarden\WithoutDNS\Share\API;

require_once __DIR__ . DIRECTORY_SEPARATOR . ".." . DIRECTORY_SEPARATOR . "Helpers" . DIRECTORY_SEPARATOR . "Log.php";

use ModulesGarden\WithoutDNS\Share\Helpers\Log;

class WithoutDNS
{
    public const API_URL = 'https://withoutdns.com/api';

    private $baseUrl;
    private $apiToken;

    public function __construct($baseUrl, $apiToken) {
        $this->baseUrl = rtrim($baseUrl, '/');
        $this->apiToken = $apiToken;
    }

    private function request($method, $endpoint, $data = []) {
        $url = $this->baseUrl . $endpoint;
        $curl = curl_init();

        $options = [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_CUSTOMREQUEST => $method,
            CURLOPT_HTTPHEADER => [
                'Authorization: Bearer ' . $this->apiToken,
                'Content-Type: application/json'
            ],
        ];

        if (!empty($data)) {
            $options[CURLOPT_POSTFIELDS] = json_encode($data);
        }

        curl_setopt_array($curl, $options);
        $response = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if (curl_errno($curl)) {
            Log::log(curl_error($curl),$endpoint);

            throw new \Exception('Curl error: ' . curl_error($curl));
        }

        curl_close($curl);

        $response = json_decode($response, true);

        $response['status_code'] = $httpCode;

        Log::log('API Request was successful',$endpoint, $response);

        return $response;
    }

    /**
     * @throws \Exception
     */
    public function getServices() {
        return $this->request('GET', '/services');
    }

    /**
     * @throws \Exception
     */
    public function getSites($serviceId = null) {
        $endpoint = '/sites' . ($serviceId ? "?service-id=$serviceId" : '');
        return $this->request('GET', $endpoint);
    }

    /**
     * @throws \Exception
     */
    public function createSite($data) {
        return $this->request('POST', '/sites', $data);
    }

    /**
     * @throws \Exception
     */
    public function createTrial($data) {
        return $this->request('POST', '/sites/trial', $data);
    }

    /**
     * @throws \Exception
     */
    public function deleteSite($siteId) {
        return $this->request('DELETE', '/sites?id=' . $siteId);
    }

    /**
     * @throws \Exception
     */
    public function getDomains() {
        return $this->request('GET', '/domains');
    }
}