<?php

namespace ModulesGarden\WithoutDNS\Share\Helpers;

require_once __DIR__ . DIRECTORY_SEPARATOR . "Database.php";

class Configuration
{
    public const API_TOKEN = 'apiToken';
    public const TEMP_URL_DURATION = 'tempUrlDuration';
    public const TEMP_DOMAIN = 'tempDomain';
    public const NUMBER_OF_DAYS_FOR_LOG_DELETE = 'daysToDelete';
    private const METHOD = 'AES-256-CBC';

    private const ENCRYPTED_CONFIGS = [
        self::API_TOKEN
    ];

    public static function getEncryptedConfigs(): array
    {
        return self::ENCRYPTED_CONFIGS;
    }

    public static function getMethod(): string
    {
        return self::METHOD;
    }

    public static function getConfig($name)
    {
        return self::getEncryptionConfig($name);
    }

    private static function getEncryptionConfig($name)
    {
        $config = parse_ini_file('/usr/local/cpanel/etc/WithoutDNS.ini', true);

        if(!$config)
        {
            $config = parse_ini_string(shell_exec("/usr/local/cpanel/3rdparty/bin/perl /usr/local/cpanel/share/WithoutDNS/Wrapper/wrapper.pl getConfiguration"));
        }

        return $config[$name] ?? '';
    }

    public static function encrypt($data): string
    {
        $iv = hex2bin(self::getEncryptionConfig('ENCRYPTION_IV')); // Convert hex IV to raw bytes
        return openssl_encrypt($data, self::METHOD, self::getEncryptionConfig('ENCRYPTION_KEY'), 0, $iv);
    }

    public static function decrypt($data): string
    {
        $iv = hex2bin(self::getEncryptionConfig('ENCRYPTION_IV')); // Convert hex IV to raw bytes
        return openssl_decrypt($data, self::METHOD, self::getEncryptionConfig('ENCRYPTION_KEY'), 0, $iv);
    }

    public static function getAll(): array
    {
        $database = new Database();
        $query = "SELECT * FROM configuration;";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->execute();
        $rows = $stmt->fetchAll($database->getConnection()::FETCH_ASSOC);

        $result = [];
        foreach ($rows as $row) {
            $result[$row['name']] = in_array($row['name'], self::ENCRYPTED_CONFIGS) ? self::decrypt($row['value']) : $row['value'];
        }

        return $result;
    }

    public static function updateOrCreate($name, $value)
    {
        $database = new Database();
        $encryptedValue = in_array($name, self::ENCRYPTED_CONFIGS) ? self::encrypt($value) : $value;

        $query = "SELECT * FROM configuration WHERE name = :name;";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(":name", $name);
        $stmt->execute();

        $existingConfig = $stmt->fetch($database->getConnection()::FETCH_ASSOC);

        if ($existingConfig) {
            $query = "UPDATE configuration SET value = :value WHERE name = :name;";
        } else {
            $query = "INSERT INTO configuration (name, value) VALUES (:name, :value);";
        }

        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(":name", $name);
        $stmt->bindParam(":value", $encryptedValue);
        $stmt->execute();
    }

    public static function getByName($name, $default = null)
    {
        $database = new Database();
        $query = "SELECT value FROM configuration WHERE name = :name;";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(":name", $name);
        $stmt->execute();

        $result = $stmt->fetch($database->getConnection()::FETCH_ASSOC);
        return $result ? (in_array($name, self::ENCRYPTED_CONFIGS) ? self::decrypt($result['value']) : $result['value']) : $default;
    }
}