<?php

namespace ModulesGarden\WithoutDNS\Share\Helpers;

require_once __DIR__ . DIRECTORY_SEPARATOR . "Database.php";

class Log
{
    public static function log($message, $action = '', $data = []): void
    {
        $database = new Database();
        $query = "INSERT INTO logs (date, message, action, data) VALUES (NOW(), :message, :action, :data)";
        $data = json_encode((array) $data);
        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindParam(":message", $message);
        $stmt->bindParam(":action", $action);
        $stmt->bindParam(":data", $data);
        $stmt->execute();
    }

    public static function getLogs()
    {
        $database = new Database();

        $query = "SELECT * FROM logs ORDER BY id DESC;";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->execute();

        $rows = $stmt->fetchAll($database->getConnection()::FETCH_ASSOC);

        // Process each row to unserialize and decode the 'data' column
        $processedRows = [];
        foreach ($rows as $row) {
            if (isset($row['data'])) {
                $decodedData = json_decode($row['data'], true);

                if ($decodedData === null && json_last_error() !== JSON_ERROR_NONE) {
                    $decodedData = [];
                }

                if (!empty($decodedData))
                {
                    $row['data'] = print_r($decodedData,true);
                }else{
                    $row['data'] = '';
                }

            }

            // Add the processed row to the result
            $processedRows[] = $row;
        }

        return $processedRows;
    }

    public static function deleteLog($id)
    {
        $database = new Database();

        $query = "DELETE FROM logs WHERE id=:id";
        $stmt = $database->getConnection()->prepare($query);
        $stmt->bindValue(':id', $id);

        $stmt->execute();
    }

    public static function deleteLogsOlderThanDays($days)
    {
        // Create a new database connection
        $database = new Database();

        // Calculate the date threshold (current date minus the specified number of days)
        $dateThreshold = date('Y-m-d H:i:s', strtotime("-$days days"));

        // Prepare the SQL query to delete logs older than the calculated date
        $query = "DELETE FROM logs WHERE date < :dateThreshold";
        $stmt = $database->getConnection()->prepare($query);

        // Bind the date threshold to the query
        $stmt->bindValue(':dateThreshold', $dateThreshold);

        // Execute the query
        $stmt->execute();

        // Optionally, return the number of rows affected
        return $stmt->rowCount();
    }
}