<?php

namespace ModulesGarden\WithoutDNS\WHM\Controllers;

require_once __DIR__ . DIRECTORY_SEPARATOR . "BaseController.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "API" . DIRECTORY_SEPARATOR . "WithoutDNS.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "Helpers" . DIRECTORY_SEPARATOR . "Log.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "Helpers" . DIRECTORY_SEPARATOR . "Configuration.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "Helpers" . DIRECTORY_SEPARATOR . "JSON.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'whostmgr' . DIRECTORY_SEPARATOR . 'docroot' . DIRECTORY_SEPARATOR . "cgi" . DIRECTORY_SEPARATOR . "WithoutDNS" . DIRECTORY_SEPARATOR ."Helpers" . DIRECTORY_SEPARATOR ."Lang.php";

use ModulesGarden\WithoutDNS\WHM\Helpers\Lang;
use ModulesGarden\WithoutDNS\Share\API\WithoutDNS;
use ModulesGarden\WithoutDNS\Share\Helpers\Log;
use ModulesGarden\WithoutDNS\Share\Helpers\Configuration;
use ModulesGarden\WithoutDNS\Share\Helpers\JSON;

class APIConfigurationController extends BaseController
{
    protected $lang;
    public function __construct()
    {
        $this->lang = new Lang();
        $this->templateName = "apiConfiguration";
    }

    protected function getVariables() :array
    {
        $configuration = Configuration::getAll();

        if (isset($configuration[Configuration::API_TOKEN]))
        {
            $api = new WithoutDNS(WithoutDNS::API_URL, $configuration[Configuration::API_TOKEN]);

            if ($this->testConnection($api))
            {
                $sourceDomains = $this->getSourceDomains($api);

                return [
                    'configuration' => $configuration,
                    'sourceDomains' => $sourceDomains,
                    'durationOptions' => $this->getDurationOptions(),
                ];
            }
        }

        return [
            'configuration' => $configuration
        ];
    }

    public function saveChangesAjax()
    {
        $formData = $_POST['formData'];
        try {
            foreach ($formData as $key=>$value)
            {
                Configuration::updateOrCreate($key,$value);
            }

            return JSON::jsonEncode([
                'message' => $this->lang->get('changes_saved_successfully')
            ]);
        }catch (\Exception $e)
        {
            return JSON::jsonEncode([
                'error' => $e->getMessage()
            ],true);
        }
    }

    public function testConnectionAjax()
    {
        $formData = $_POST['formData'];

        $apiToken = key_exists(Configuration::API_TOKEN,$formData) ? $formData[Configuration::API_TOKEN] : '';

        $api = new WithoutDNS(WithoutDNS::API_URL, $apiToken);

        if ($this->testConnection($api))
        {
            foreach ($formData as $key=>$value)
            {
                Configuration::updateOrCreate($key,$value);
            }

            Log::log($this->lang->get('connection_established_successfully'),'Test Connection');

            return JSON::jsonEncode([
                'message' => $this->lang->get('connection_established_successfully')
            ]);
        }

        return JSON::jsonEncode([
            'error' => $this->lang->get('connection_can_not_be_established')
        ],true);
    }

    public function testConnection(WithoutDNS $api): bool
    {
        try {
            //try to connect to API
            $result = $api->getServices();

            if (key_exists('status',$result) && $result['status'] === 'success') {
                return true;
            }else{
                Log::log($this->lang->get('connection_can_not_be_established'),'Test Connection',$result);
            }
        } catch (\Exception $e) {
            Log::log($e->getMessage(),'Test Connection');
        }

        return false;
    }

    public function getSourceDomains(WithoutDNS $api): array
    {
        $data = [];

        try {
            $result = $api->getDomains();

            if (key_exists('status', $result) && $result['status'] == 'success') {
                foreach ($result['data']['records'] as $domain) {
                    $data[$domain['name']] = $domain['name']; // Key and value are the same
                }
            }else{
                Log::log($this->lang->get('failed_to_load_source_domains'),'Load Source Domains',$result);
            }
        } catch (\Exception $e) {
            Log::log($e->getMessage(),'Load Source Domains');
        }

        return $data;
    }

    public function getDurationOptions(): array
    {
        return [
            '15m' => $this->lang->get('15m'),
            '1h' => $this->lang->get('1h'),
            '24h' => $this->lang->get('24h'),
            '7d' => $this->lang->get('7d'),
            '30d' => $this->lang->get('30d'),
            'never' => $this->lang->get('never'),
        ];
    }
}