#!/usr/local/cpanel/3rdparty/bin/php -q
<?php

require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "Helpers" . DIRECTORY_SEPARATOR . "Log.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "Helpers" . DIRECTORY_SEPARATOR . "DomainsUrls.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "API" . DIRECTORY_SEPARATOR . "WithoutDNS.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "API" . DIRECTORY_SEPARATOR . "WHM.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "Helpers" . DIRECTORY_SEPARATOR . "Configuration.php";
require_once DIRECTORY_SEPARATOR . 'usr' . DIRECTORY_SEPARATOR . 'local' . DIRECTORY_SEPARATOR . 'cpanel' . DIRECTORY_SEPARATOR . 'share' . DIRECTORY_SEPARATOR . 'WithoutDNS' . DIRECTORY_SEPARATOR . "Helpers" . DIRECTORY_SEPARATOR . "DomainsHelper.php";

use ModulesGarden\WithoutDNS\Share\API\WHM;
use ModulesGarden\WithoutDNS\Share\Helpers\Configuration;
use ModulesGarden\WithoutDNS\Share\Helpers\Log;
use ModulesGarden\WithoutDNS\Share\Helpers\DomainsUrls;
use ModulesGarden\WithoutDNS\Share\API\WithoutDNS;
use ModulesGarden\WithoutDNS\Share\Helpers\DomainsHelper;

try
{
    $input = (count($argv) > 1) ? $argv : array();

    $data = readEventData();

    if (in_array('--describe', $input))
    {
        echo json_encode(describe());
        exit;
    }

    if(in_array('--terminate', $input))
    {
        try
        {
            $username = $data['data']['user'];

            $domains = $whmAPI = new WHM();
            $domains = $whmAPI->getUserDomains($username);

            foreach($domains as $domain)
            {
                handleDomainDeletion($domain,'terminate',$data);
            }
        }
        catch(Exception $e)
        {
            Log::log($e->getMessage());
        }

        $result = [1,'success'];
        echo trim(json_encode($result));
        die;
    }

    if(in_array('--suspendacct', $input))
    {
        try
        {
            $username = $data['data']['args']['user'];

            $domains = $whmAPI = new WHM();
            $domains = $whmAPI->getUserDomains($username);

            foreach($domains as $domain)
            {
                handleDomainDeletion($domain,'suspend account',$data);
            }
        }
        catch(Exception $e)
        {
            Log::log($e->getMessage());
        }

        $result = [1,'success'];
        echo trim(json_encode($result));
        die;
    }

}
catch (\Throwable $th)
{
    $result = [0, $th->getMessage(), $th->getTrace()];
    echo trim(json_encode($result));
    die;
}

function describe()
{
    return [
        [
            'category' => 'Whostmgr',
            'event'    => 'Accounts::Remove',
            'stage'    => 'pre',
            'hook'     => '/usr/local/cpanel/whostmgr/docroot/cgi/WithoutDNS/hooks.php --terminate',
            'exectype' => 'script',
            'weight'   => 999,
            'blocking' => 1,
        ],
        [
            'category' => 'Whostmgr',
            'event'    => 'Accounts::suspendacct',
            'stage'    => 'post',
            'hook'     => '/usr/local/cpanel/whostmgr/docroot/cgi/WithoutDNS/hooks.php --suspendacct',
            'exectype' => 'script',
            'weight'   => 999,
            'blocking' => 1,
        ],
    ];
}

function handleDomainDeletion($domain, $action, $logData)
{
    $domainUrlData = DomainsUrls::getByDomainName($domain);
    if ($domainUrlData) {
        DomainsUrls::deleteByDomainName($domain);

        $api = new WithoutDNS(
            WithoutDNS::API_URL,
            Configuration::getByName(Configuration::API_TOKEN, '')
        );

        if (isset($domainUrlData['data']['model']['id'])) {
            try {
                $api->deleteSite($domainUrlData['data']['model']['id']);
            } catch (Exception $e) {
                Log::log('Site delete resulted with an error: ' . $e->getMessage(), $action, $logData);
            }
        }

        Log::log('Site deleted successfully!', $action, $logData);
    }
}

function readEventData()
{
    $raw_data = '';
    $stdin_fh = fopen('php://stdin', 'r');
    if (is_resource($stdin_fh)) {
        stream_set_blocking($stdin_fh, 0);
        while (($line = fgets($stdin_fh, 1024)) !== false) {
            $raw_data .= trim($line);
        }
        fclose($stdin_fh);
    }

    if ( ! empty($raw_data)) {
        $input_data = json_decode($raw_data, true);
    } else {
        $input_data = array('context' => array(), 'data' => array(), 'hook' => array());
    }

    return $input_data;
}
