<?php

require_once __DIR__ . DIRECTORY_SEPARATOR . "Share/Helpers/Database.php";
require_once __DIR__ . DIRECTORY_SEPARATOR . "Share/Helpers/Configuration.php";

use ModulesGarden\WithoutDNS\Share\Helpers\Database;
use ModulesGarden\WithoutDNS\Share\Helpers\Configuration;

try {
    $database = new Database();
    $conn = $database->getConnection();

    // Fetch all encrypted configuration values
    $query = "SELECT name, value FROM configuration;";
    $stmt = $conn->prepare($query);
    $stmt->execute();
    $rows = $stmt->fetchAll($conn::FETCH_ASSOC);

    foreach ($rows as $row) {
        $name = $row['name'];
        $encryptedValue = $row['value'];

        // Skip if this setting is not an encrypted field
        if (!in_array($name, Configuration::getEncryptedConfigs())) {
            continue;
        }

        // Try decrypting using the old incorrect IV format (hex string IV)
        $ivHex = Configuration::getConfig('ENCRYPTION_IV');
        $ivRaw = hex2bin($ivHex); // Correct IV format
        $key = Configuration::getConfig('ENCRYPTION_KEY');

        $decryptedValue = openssl_decrypt($encryptedValue, Configuration::getMethod(), $key, 0, $ivHex);

        // If old method fails, try the correct IV format
        if ($decryptedValue === false) {
            echo "Skipping $name: Cannot decrypt with old method.\n";
            continue;
        }

        // Re-encrypt using the correct IV format (raw bytes IV)
        $newEncryptedValue = openssl_encrypt($decryptedValue, Configuration::getMethod(), $key, 0, $ivRaw);

        // Update the database with the corrected encryption
        $updateQuery = "UPDATE configuration SET value = :value WHERE name = :name;";
        $updateStmt = $conn->prepare($updateQuery);
        $updateStmt->bindParam(":value", $newEncryptedValue);
        $updateStmt->bindParam(":name", $name);
        $updateStmt->execute();

        echo "Updated: $name\n";
    }

    echo "Migration complete!\n";
}catch (\Exception $exception){
    echo $exception->getMessage();
}

